@@ -63,11 +63,11 @@ gok_output_update_predictions (GokOutput    *output,
 						      prediction_list);
 			break;
 		case OUTPUT_KEYSTRING:
-			/* TODO: Test! */
 			string = (gchar *) data;
 			if (string && g_utf8_validate (string, -1, NULL)) 
 			{
-				while (string) 
+			        letter = g_utf8_get_char (string);
+				while (letter && string) 
 				{
 					if (prediction_list)
 					{
@@ -76,12 +76,13 @@ gok_output_update_predictions (GokOutput    *output,
 							g_free (prediction_list[i]);
 						g_free (prediction_list);
 					}
-					letter = g_utf8_get_char (string);
+
 					prediction_list = gok_wordcomplete_process_and_predict (
 						gok_wordcomplete_get_default (),
 						letter,
 						gok_data_get_num_predictions ());
 					string = g_utf8_next_char (string);
+					letter = g_utf8_get_char (string);
 				}
 				gok_keyboard_set_predictions (gok_main_get_current_keyboard (), 
 							      prediction_list);
@@ -222,14 +223,17 @@ GokOutput* gok_output_new_from_xml (xmlNode* pNode)
 		typeOutput = OUTPUT_KEYSYM;
 		flagOutput = SPI_KEY_PRESSRELEASE;
 	}
-	else if (xmlStrcmp (pStringAttributeValue, (const xmlChar *)"exec")
-		== 0)
+	else if (xmlStrcmp (pStringAttributeValue, (const xmlChar *)"keystring") == 0)
+	{
+		typeOutput = OUTPUT_KEYSTRING;
+		flagOutput = SPI_KEY_PRESSRELEASE;
+	}
+	else if (xmlStrcmp (pStringAttributeValue, (const xmlChar *)"exec") == 0)
 	{
 		typeOutput = OUTPUT_EXEC;
 		flagOutput = SPI_KEY_PRESSRELEASE;
 	}
-	else if (xmlStrcmp (pStringAttributeValue, (const xmlChar *)"internal")
-		== 0)
+	else if (xmlStrcmp (pStringAttributeValue, (const xmlChar *)"internal") == 0)
 	{
 		typeOutput = OUTPUT_INTERNAL;
 		flagOutput = SPI_KEY_PRESSRELEASE;  /* not necessary? */
@@ -315,7 +319,15 @@ void gok_output_send_to_system (GokOutput* pOutput, gboolean bWordCompletion)
 	{
 		if (pOutput->Type == OUTPUT_KEYSYM)
 		{
-			keysym = XStringToKeysym (pOutput->Name);
+		        if (g_str_has_prefix (pOutput->Name, "U+"))
+		        {
+			        keysym = 0x01000000 |
+				    g_ascii_strtoull ((gchar *) (pOutput->Name + 2), NULL, 16);
+		        } 
+		        else
+		        {
+			        keysym = XStringToKeysym (pOutput->Name);
+		        }
 			
 			gok_log ("Sending a keysym");
 			gok_log ("pOutput->Name = %s", pOutput->Name);
@@ -385,11 +397,28 @@ void gok_output_send_to_system (GokOutput* pOutput, gboolean bWordCompletion)
 		}
 		else if (pOutput->Type == OUTPUT_KEYSTRING)
 		{
+			Accessible *focussed_object = gok_spy_get_focussed_object ();
+			AccessibleStateSet *state = NULL;
 			gok_log ("Sending a string");
 			gok_log ("pOutput->Name = SPI_KEY_STRING = %s", pOutput->Name);
-
-			SPI_generateKeyboardEvent (0, pOutput->Name, SPI_KEY_STRING);
-
+			
+			if (focussed_object && Accessible_isEditableText (focussed_object) &&
+				(state = Accessible_getStateSet (focussed_object)) && 
+				AccessibleStateSet_contains (state, SPI_STATE_FOCUSED)) 
+			{
+				AccessibleText *text = Accessible_getText (focussed_object);
+				AccessibleEditableText *editable = Accessible_getEditableText (focussed_object);
+				AccessibleEditableText_insertText (editable, AccessibleText_getCaretOffset (text),
+								   pOutput->Name, 
+								   strlen (pOutput->Name));
+				AccessibleText_unref (text);
+				AccessibleEditableText_unref (editable);
+			}
+			else
+			{
+			    SPI_generateKeyboardEvent (0, pOutput->Name, SPI_KEY_STRING);
+			}
+			if (state) AccessibleStateSet_unref (state);
 			/* update the word prediction keys with the new output */
 			if (bWordCompletion == TRUE && gok_output_can_predict ())
 			{
