@@ -32,6 +32,7 @@ extern "C" {
 #endif /* __cplusplus */
 
 typedef enum {
+        OUTPUT_INVALID,
 	OUTPUT_KEYSYM,
 	OUTPUT_KEYCODE,
 	OUTPUT_KEYSTRING,
@@ -43,7 +44,7 @@ typedef enum {
 typedef struct GokOutput{
 	gint Type;
 	AccessibleKeySynthType Flag;
-	gchar* Name;
+	const gchar* Name;
 	struct GokOutput* pOutputNext;	
 } GokOutput;
 
