@@ -115,6 +115,8 @@ static Accessible* m_ptheAccessibleWithText = NULL;
 
 static Accessible* theContextMenuAccessible = NULL;
 
+static Accessible* currently_focussed_object = NULL;
+
 static AccessibleEventListener* focusListener;
 static AccessibleEventListener* defunctnessListener;
 static AccessibleDeviceListener* mouseListener;
@@ -132,6 +134,24 @@ static gboolean spyshutdown = FALSE;/* this global could be removed if we have a
 
 #define MAX_CHILD_COUNT 100
 
+
+static void
+gok_spy_set_focussed_object (Accessible *accessible)
+{
+    currently_focussed_object = accessible;
+}
+
+/*
+ * gok_spy_get_focussed_object:
+ *
+ * returns: the currently focussed object, or NULL if no object has emitted a focus event.
+ */
+Accessible *
+gok_spy_get_focussed_object (void)
+{
+    return currently_focussed_object;
+}
+
 /* 
  * for debugging
  */
@@ -1194,6 +1214,7 @@ gok_spy_is_ui (Accessible *accessible, AccessibleRole role)
 	case SPI_ROLE_PAGE_TAB:
 	case SPI_ROLE_TOGGLE_BUTTON:
 	case SPI_ROLE_SLIDER:
+	case SPI_ROLE_SCROLL_BAR:
 		interesting = TRUE;
 		break;
 	default:
@@ -1396,7 +1417,10 @@ GSList* gok_spy_append_node (GSList* nodes,
 	AccessibleNode* pNewNode = NULL;
 	AccessibleRelation** relations = NULL;
 	Accessible* targetAccessible = NULL;
+	AccessibleRole role;
 	AccessibleRelationType type = -1;
+	AccessibleStateSet *states;
+
 	char* pName = NULL, *s;
 	int i = 0;
 	int j = 0;
@@ -1532,6 +1556,11 @@ GSList* gok_spy_append_node (GSList* nodes,
 			}
 		}
 
+		if (pName == NULL) 
+		{
+		    role = Accessible_getRole (pAccessible);
+		}
+
 		/* 
 		 * Role "TABLE-CELL" is used for 'generic' table children. 
 		 * These children's behavior is governed more by the fact that they are
@@ -1542,9 +1571,23 @@ GSList* gok_spy_append_node (GSList* nodes,
 		 * when possible.  In the case of GtkTreeviews in particular, where row-selection
 		 * is the dominant user activity, this is especially appropriate.
 		 */
-		if (pName == NULL && (Accessible_getRole (pAccessible) == SPI_ROLE_TABLE_CELL)) {
+		if (pName == NULL && role == SPI_ROLE_TABLE_CELL) {
 			pName = gok_spy_concatenate_child_names (pAccessible);
 		}
+		else if (pName == NULL && role == SPI_ROLE_SCROLL_BAR) {
+		    states = Accessible_getStateSet (pAccessible);
+		    if (AccessibleStateSet_contains (states, SPI_STATE_VERTICAL))
+		    {
+			/* translators: abbreviated version of "Vertical Scrollbar" */
+			pName = g_strdup (_("V Scrollbar"));
+		    }
+		    else
+		    {
+			/* translators: abbreviated version of "Horizontal Scrollbar" */
+			pName = g_strdup (_("H Scrollbar"));
+		    }
+		    AccessibleStateSet_unref (states);
+		}
 		else if (pName == NULL) { /* desperate attempt to reuse singleton-child's name */
 			Accessible *tmp_parent = pAccessible;
 			gok_log("no description... checking for singleton child with name.");
@@ -1615,8 +1658,8 @@ GSList* gok_spy_append_node (GSList* nodes,
 			}
 			if (pName == NULL)
 			{
-				/* TODO: I18N mark this string for translation! */
-				pName = g_strdup ("Table");
+			        /* translators: "table" as in row/column data structure */
+				pName = g_strdup (_("Table"));
 			}
 		}			
 
@@ -1710,6 +1753,7 @@ void gok_spy_focus_listener (const AccessibleEvent *event, void *user_data)
 	EventNode* en; 
 	gok_log_enter();
 	AccessibleEvent_ref (event);
+	gok_spy_set_focussed_object (event->source);
 	en = (EventNode*)g_malloc(sizeof(EventNode));
 	en->event = event;
 	en->type = GOKSPY_FOCUS_EVENT;
