@@ -504,7 +504,16 @@ utf8_add_to_list_from_lines (GSList *list, gchar **lines, gboolean read_freqs)
 		}
 		else
 		{
-		        tokens = g_strsplit (*lines, "/", 2);
+			if (strchr (*lines, '/'))
+			{
+				g_strdelimit (*lines, "\t\\", '/');
+				tokens = g_strsplit (*lines, "/", 2);
+			}
+			else
+			{
+				g_strdelimit (*lines, "\t\\", ' ');
+				tokens = g_strsplit (*lines, " ", 2);
+			}
 			string = tokens ? tokens[0] : NULL;
 		}
 
