@@ -131,6 +131,7 @@ static gboolean gok_main_check_sticky_keys (GtkWidget *widget);
 static char* accessmethodname = NULL;
 static char* inputdevicename = NULL;
 static char* mainkeyboardname = NULL;
+static char* custom_compose_kbd_name = NULL;
 static int bDisplayKeyboardEditor = 0;
 static char *geometry = NULL;
 static int geometry_bitmask;
@@ -268,6 +269,12 @@ on_gok_window_state_event ( GtkWidget       *widget,
 	return FALSE;
 }
 
+const gchar *
+gok_main_get_custom_compose_kbd_name ()
+{
+	return custom_compose_kbd_name;
+}
+
 /**
 * main
 * @argc: ignored
@@ -1440,6 +1447,84 @@ gok_main_read_rc ()
     }
 }
 
+static GokKeyboard*
+gok_append_keyboard_from_file (GokKeyboard *keyboard, const gchar *complete_path)
+{
+    GokKeyboard *new = gok_keyboard_read (complete_path);
+
+    if (new) 
+    {
+	new->pKeyboardPrevious = keyboard;
+	if (keyboard)
+	{
+	    new->pKeyboardNext = keyboard->pKeyboardNext;
+	    keyboard->pKeyboardNext = new;
+	}
+    
+	/* all predefined keyboards are assumed to be laid out */
+	new->bRequiresLayout = FALSE;
+	new->bLaidOut = TRUE;
+	keyboard = new;
+    }
+
+    return keyboard;
+}
+
+static GokKeyboard*
+gok_main_create_compose_keyboards ()
+{
+    GokKeyboard *keyboard = NULL, *prev_kbd = NULL;
+    GokKeyboard *first = NULL;
+
+    /* create the default "core" keyboard (aka qwerty keyboard) */
+    if (gok_data_get_use_xkb_kbd ()) {
+	keyboard = first = gok_keyboard_get_core ();
+	if (keyboard)
+	    keyboard->pKeyboardPrevious = NULL;
+	else
+	    gok_data_set_use_xkb_kbd (FALSE);
+
+	prev_kbd = keyboard;
+    }
+    
+    /* create the Alpha and Alpha-Freq keyboards */
+    keyboard = gok_keyboard_get_alpha ();
+    if (prev_kbd) 
+    {
+	prev_kbd->pKeyboardNext = keyboard;
+    }
+    if (keyboard)
+    {
+	keyboard->pKeyboardPrevious = prev_kbd;
+	prev_kbd = keyboard;
+	if (first == NULL) first = keyboard;
+    }
+    keyboard = gok_keyboard_get_alpha_by_frequency ();
+    if (prev_kbd) 
+    {
+	prev_kbd->pKeyboardNext = keyboard;
+    }
+    if (keyboard)
+    {
+	keyboard->pKeyboardPrevious = prev_kbd;
+	prev_kbd = keyboard;
+	if (first == NULL) 
+	    first = keyboard;
+    }
+    
+    /* read the user-defined compose keyboard, if one is specified */
+    if (gok_data_get_compose_keyboard_type () == GOK_COMPOSE_CUSTOM) 
+    {
+	keyboard = gok_append_keyboard_from_file (keyboard, 
+						  gok_data_get_custom_compose_filename ());
+	custom_compose_kbd_name = keyboard->Name;
+	if (first == NULL)
+	    first = keyboard;
+    }
+    
+    return first;
+}
+
 /**
 * gok_main_read_keyboards:
 *
@@ -1449,6 +1534,7 @@ gok_main_read_rc ()
 */
 void gok_main_read_keyboards ()
 {
+    GokKeyboard *keyboard;
     GConfClient *gconf_client = NULL;
     GError *gconf_err = NULL;
     gchar *directory_name = NULL;
@@ -1456,6 +1542,15 @@ void gok_main_read_keyboards ()
 	
     gconf_client = gconf_client_get_default ();
     
+    /* create the compose keyboards first... */
+    keyboard = m_pKeyboardFirst = gok_main_create_compose_keyboards ();
+
+    if (m_pKeyboardFirst == NULL)
+    {
+	gok_main_display_error (_("Can't create a compose keyboard!"));
+	exit (1);
+    }
+
     directory_name = gconf_client_get_string (gconf_client,
 					      GOK_GCONF_KEYBOARD_DIRECTORY,
 					      &gconf_err);
@@ -1473,9 +1568,21 @@ void gok_main_read_keyboards ()
     }
     else
     {
-	/* read the keyboard files from directory_name */
+	/* read the keyboard files from system directory */
 	gok_modifier_open();
-	m_pKeyboardFirst = gok_main_read_keyboards_from_dir (directory_name);
+	keyboard = gok_main_read_keyboards_from_dir (directory_name, keyboard);
+    }
+
+    g_free (directory_name);
+    
+    directory_name = gconf_client_get_string (gconf_client,
+					      GOK_GCONF_AUX_KEYBOARD_DIRECTORY,
+					      &gconf_err);
+
+    if ((directory_name != NULL) && (strlen (directory_name) > 0) && (gconf_err == NULL))
+    {
+	/* read the keyboard files from directory_name */
+	keyboard = gok_main_read_keyboards_from_dir (directory_name, keyboard);
     }
 
     g_free (directory_name);
@@ -1492,27 +1599,23 @@ void gok_main_read_keyboards ()
 * @directory: The name of the keyboard root directory location.
 * 
 * Reads all the keyboard files from the given directory or a subdirectory that
-* is a closer match based on user's language if it exists.
+* is a closer match based on user's language if it exists.  
 *
 * Returns: A pointer to the first keyboard, NULL if no keyboards could be read.
 */
-GokKeyboard* gok_main_read_keyboards_from_dir (const char *directory)
+GokKeyboard* 
+gok_main_read_keyboards_from_dir (const char *directory, GokKeyboard *keyboard)
 {
-	GokKeyboard* pKeyboardFirst;
-	GokKeyboard* pKeyboard;
-	GokKeyboard* pKeyboardPrevious;
+	GokKeyboard *pKeyboardFirst = keyboard;
 	DIR* pDirectoryKeyboards;
 	struct dirent* pDirectoryEntry;
 	gchar* filename;
 	gchar* complete_path;
-	GList* langlist;
+	GList* langlist = NULL;
 	gchar dirtemp[1024]; /* no reliable way to avoid magic # here */
 
 	gok_log_enter();
-	pKeyboardFirst = NULL;
-	pKeyboard = NULL;
-	pKeyboardPrevious = NULL;
-	langlist == NULL;
+
 	pDirectoryKeyboards = NULL;
 
 	/* find the locale specific keyboards */
@@ -1527,7 +1630,6 @@ GokKeyboard* gok_main_read_keyboards_from_dir (const char *directory)
 			pDirectoryKeyboards = opendir (dirtemp);
 			i++;
 		}
-		g_list_free(langlist);
 	}
 
 	if (pDirectoryKeyboards == NULL) { /* no locale specific directory found */
@@ -1546,59 +1648,29 @@ GokKeyboard* gok_main_read_keyboards_from_dir (const char *directory)
 		return FALSE;
 	}	
 
-	/* create the default "core" keyboard (aka qwerty keyboard) */
-	if (gok_data_get_use_xkb_kbd ()) {
-		pKeyboard = pKeyboardFirst = gok_keyboard_get_core ();
-		if (pKeyboard)
-			pKeyboard->pKeyboardPrevious = NULL;
-		else
-		        gok_data_set_use_xkb_kbd (FALSE);
-		pKeyboardPrevious = pKeyboard;
+	/* seek to the end of the list before appending */
+	while (keyboard && keyboard->pKeyboardNext)
+	{
+	    keyboard = keyboard->pKeyboardNext;
 	}
 
-	/* create the Alpha and Alpha-Freq keyboards */
-	pKeyboard = gok_keyboard_get_alpha ();
-	pKeyboard->pKeyboardNext = pKeyboardPrevious->pKeyboardNext;
-	pKeyboardPrevious->pKeyboardNext = pKeyboard;
-	pKeyboard->pKeyboardPrevious = pKeyboardPrevious;
-	pKeyboardPrevious = pKeyboard;
-	pKeyboard = gok_keyboard_get_alpha_by_frequency ();
-	pKeyboard->pKeyboardNext = pKeyboardPrevious->pKeyboardNext;
-	pKeyboardPrevious->pKeyboardNext = pKeyboard;
-	pKeyboard->pKeyboardPrevious = pKeyboardPrevious;
-	pKeyboardPrevious = pKeyboard;	
-
 	/* look at each file in the directory */
 	while ((pDirectoryEntry = readdir (pDirectoryKeyboards)) != NULL)
 	{
 		/* is this a keyboard file? */
 		filename = pDirectoryEntry->d_name;
 		if ( (strlen(filename) >= 4)
-		&& (strcmp(filename + (strlen(filename)-4), ".kbd") == 0) 
-		&& ((strlen(filename) < 12) ||
-		    (strcmp(filename + (strlen(filename)-12), 
-			    "Keyboard.kbd") != 0) || 
-		    (gok_data_get_use_xkb_kbd () == FALSE)))
+		     && (strcmp(filename + (strlen(filename)-4), ".kbd") == 0))
 		{
-			/* read the keyboard file */
+			/* read the keyboard file and add to list */
                         complete_path = g_build_filename (directory, filename,
                                                           NULL);
 			gok_log ("complete_path = %s", complete_path);
-			pKeyboard = gok_keyboard_read (complete_path);
+			keyboard = gok_append_keyboard_from_file (keyboard, complete_path);
 			g_free (complete_path);
 			
-			/* add this keyboard into the list of keyboards */
-			pKeyboard->pKeyboardPrevious = pKeyboardPrevious;
 			if (pKeyboardFirst == NULL)
-				pKeyboardFirst = pKeyboard;
-			if (pKeyboardPrevious != NULL)
-				pKeyboardPrevious->pKeyboardNext = pKeyboard;
-			
-			/* all predefined keyboards are assumed to be laid out */
-			pKeyboard->bRequiresLayout = FALSE;
-			pKeyboard->bLaidOut = TRUE;
-			
-			pKeyboardPrevious = pKeyboard;
+				pKeyboardFirst = keyboard;
 		}
 	}
 	
