@@ -36,7 +36,10 @@ extern "C" {
 GtkWidget* gok_main_create_window (gboolean is_dock);
 void gok_main_set_cursor (GdkCursor *cursor);
 gint gok_main_open(gint argc, gchar *argv[]);
-gboolean gok_main_display_scan (GokKeyboard* pKeyboard, char* NameKeyboard, KeyboardTypes keyboardType, KeyboardLayouts layoutType);
+void gok_main_raise_window (void);
+const gchar *gok_main_get_custom_compose_kbd_name (void);
+GokKeyboard *gok_main_keyboard_find_byname (const gchar* NameKeyboard);
+gboolean gok_main_display_scan (GokKeyboard* pKeyboard, gchar* nameKeyboard, KeyboardTypes typeKeyboard, KeyboardLayouts layout, KeyboardShape shape);
 gboolean gok_main_display_scan_previous (void);
 gboolean gok_main_display_scan_previous_premade (void);
 gboolean gok_main_display_scan_reset (void);
@@ -50,7 +53,7 @@ void gok_main_resize_window (GtkWidget* pWindow, GokKeyboard *pKeyboard, gint Wi
 void gok_main_store_window_center (void);
 Accessible* gok_main_get_foreground_window_accessible (void);
 void gok_main_read_keyboards (void);
-GokKeyboard* gok_main_read_keyboards_from_dir (const char *directory);
+GokKeyboard* gok_main_read_keyboards_from_dir (const char *directory, GokKeyboard *first);
 void gok_main_on_window_position_change (void);
 void gok_main_get_our_window_size (gint* pWidth, gint* pHeight);
 void gok_main_display_error (gchar* ErrorString);
@@ -60,6 +63,7 @@ Display *gok_main_display (void);
 gchar* gok_main_get_access_method_override (void);
 gboolean gok_main_get_extras (void);
 gboolean gok_main_get_login (void);
+void gok_main_check_accessibility (void);
 
 /* private */
 void gok_main_app_change_listener (Accessible* pAccessible);
